# The upgrade is going to be a little messy, because golang uses
# golang to bootstrap.  The problem is that currently mageia puts a lot
# of architecture independent files in /usr/lib64.  So I'm going to have
# to compile golang using the 1.4 packages with the files in the wrong
# location, and then install the arch independent files in /usr/lib,
# then bootstrap golang with golang with the architecture independent
# files in /usr/lib

# If you want to bootstrap golang on a clean platform you can either
# build golang 1.4 or else build gcc-go

# Set to %%{_libdir}/%%{name} when bootstrapping from Mageia 1.4 packages

%global goroot_bootstrap_dir /usr/lib/%{name}

%global gopath     %{_datadir}/gocode

# build ids are not currently generated:
# https://code.google.com/p/go/issues/detail?id=5238
#
# also, debuginfo extraction currently fails with
# "Failed to write file: invalid section alignment"
%global debug_package %{nil}

# we are shipping the full contents of src in the data subpackage, which
# contains binary-like things (ELF data for tests, etc)
%global _binaries_in_noarch_packages_terminate_build 0

# Do not check any files in doc or src for requires
%global __requires_exclude_from ^(%{_datadir}|/usr/lib)/%{name}/(doc|src)/.*$

# Don't alter timestamps of especially the .a files (or else go will rebuild later)
# Actually, don't strip at all since we are not even building debug packages and this corrupts the dwarf testdata
%global __strip /bin/true

# rpmbuild magic to keep from having meta dependency on libc.so.6
%define _use_internal_dependency_generator 0
%define __find_requires %{nil}
%global __spec_install_post /usr/lib/rpm/check-rpaths   /usr/lib/rpm/check-buildroot  \
  /usr/lib/rpm/brp-compress

# Golang build options.

# Build golang using external/internal(close to cgo disabled) linking.
%global external_linker 1

# Build golang with cgo enabled/disabled(later equals more or less to internal linking).
%global cgo_enabled 1

# Use golang/gcc-go as bootstrap compiler
%global golang_bootstrap 1

# Controls what ever we fail on failed tests
%ifarch aarch64
%global fail_on_tests 0
%else
%global fail_on_tests 1
%endif

# Build golang shared objects for stdlib
%global shared 1

# Pre build std lib with -race enabled
%ifarch x86_64
%global race 1
%else
%global race 0
%endif

# Mageia GOROOT
%global goroot          /usr/lib/%{name}

%ifarch x86_64
%global gohostarch  amd64
%endif
%ifarch %{ix86}
%global gohostarch  386
%endif
%ifarch %{arm}
%global gohostarch  arm
%endif
%ifarch aarch64
%global gohostarch  arm64
%endif

%global go_api 1.8

Name:           golang
Version:        1.11.13
Release:        %mkrel 1
Summary:        The Go Programming Language
# source tree includes several copies of Mark.Twain-Tom.Sawyer.txt under Public Domain
Group:          Development/Other
License:        BSD and Public Domain
URL:            http://golang.org/
Source0:        https://storage.googleapis.com/golang/go%{version}.src.tar.gz

# The compiler is written in Go. Needs go(1.4+) compiler for build.
%if !%{golang_bootstrap}
BuildRequires:  gcc-go >= 5
%else
BuildRequires:  golang >= 1.4
%endif
BuildRequires:  net-tools

# for tests
BuildRequires:  pkgconfig(libpcre)
BuildRequires:  glibc-devel
BuildRequires:  glibc-static-devel

Provides:       go = %{version}-%{release}
Requires:       %{name}-bin = %{version}-%{release}
Requires:       %{name}-src = %{version}-%{release}

# use the arch dependent path in the bootstrap
Patch212:       golang-1.10-bootstrap-binary-path.patch

# we had been just removing the zoneinfo.zip, but that caused tests to fail for users that 
# later run `go test -a std`. This makes it only use the zoneinfo.zip where needed in tests.
#Patch215:       go1.5-zoneinfo_testing_only.patch

# Having documentation separate was broken
Obsoletes:      %{name}-docs < 1.1-4

# RPM can't handle symlink -> dir with subpackages, so merge back
Obsoletes:      %{name}-data < 1.1.1-4

# go1.4 deprecates a few packages
Obsoletes:      %{name}-vim < 1.4
Obsoletes:      emacs-%{name} < 1.4

Source100:      golang-gdbinit
Source101:      golang-prelink.conf
# let this match the macros in macros.golang
Source102:      macros.golang

%description
%{summary}.

%package       docs
Summary:       Golang compiler docs
Group:         Development/Other
Requires:      %{name} = %{version}-%{release}
BuildArch:     noarch
Obsoletes:     %{name}-docs < 1.1-4

%description   docs
%{summary}.

%package       misc
Summary:       Golang compiler miscellaneous sources
Group:         Development/Other
Requires:      %{name} = %{version}-%{release}
BuildArch:     noarch

%description   misc
%{summary}.

%package       tests
Summary:       Golang compiler tests for stdlib
Group:         Development/Other
Requires:      %{name} = %{version}-%{release}
BuildArch:     noarch

%description   tests
%{summary}.

%package        src
Summary:        Golang compiler source tree
Group:          Development/Other
BuildArch:      noarch
%description    src
%{summary}

%package        bin
Summary:        Golang core compiler tools
Group:          Development/Other
Requires:       go = %{version}-%{release}
# Pre-go1.5, all arches had to be bootstrapped individually, before usable, and
# env variables to compile for the target os-arch.
# Now the host compiler needs only the GOOS and GOARCH environment variables
# set to compile for the target os-arch.
Obsoletes:      %{name}-pkg-bin-linux-386 < 1.4.99
Obsoletes:      %{name}-pkg-bin-linux-amd64 < 1.4.99
Obsoletes:      %{name}-pkg-bin-linux-arm < 1.4.99
Obsoletes:      %{name}-pkg-linux-386 < 1.4.99
Obsoletes:      %{name}-pkg-linux-amd64 < 1.4.99
Obsoletes:      %{name}-pkg-linux-arm < 1.4.99
Obsoletes:      %{name}-pkg-darwin-386 < 1.4.99
Obsoletes:      %{name}-pkg-darwin-amd64 < 1.4.99
Obsoletes:      %{name}-pkg-windows-386 < 1.4.99
Obsoletes:      %{name}-pkg-windows-amd64 < 1.4.99
Obsoletes:      %{name}-pkg-plan9-386 < 1.4.99
Obsoletes:      %{name}-pkg-plan9-amd64 < 1.4.99
Obsoletes:      %{name}-pkg-freebsd-386 < 1.4.99
Obsoletes:      %{name}-pkg-freebsd-amd64 < 1.4.99
Obsoletes:      %{name}-pkg-freebsd-arm < 1.4.99
Obsoletes:      %{name}-pkg-netbsd-386 < 1.4.99
Obsoletes:      %{name}-pkg-netbsd-amd64 < 1.4.99
Obsoletes:      %{name}-pkg-netbsd-arm < 1.4.99
Obsoletes:      %{name}-pkg-openbsd-386 < 1.4.99
Obsoletes:      %{name}-pkg-openbsd-amd64 < 1.4.99

Requires(post): %{_sbindir}/update-alternatives
Requires(postun): %{_sbindir}/update-alternatives

# We strip the meta dependency, but go does require glibc.
# This is an odd issue, still looking for a better fix.
Requires:       glibc
Requires:       gcc
%description    bin
%{summary}

# Workaround old RPM bug of symlink-replaced-with-dir failure
%pretrans -p <lua>
for _,d in pairs({"api", "doc", "include", "lib", "src"}) do
  path = "%{goroot}/" .. d
  if posix.stat(path, "type") == "link" then
    os.remove(path)
    posix.mkdir(path)
  end
end

%if %{shared}
%package        shared
Summary:        Golang shared object libraries
Group:          Development/Other

%description    shared
%{summary}.
%endif

%if %{race}
%package        race
Summary:        Golang std library with -race enabled

Requires:       %{name} = %{version}-%{release}

%description    race
%{summary}.
%endif

%prep
%setup -q -n go

# use the arch dependent path in the bootstrap
%patch212 -p1

#%patch215 -p1

%build
# print out system information
uname -a
cat /proc/cpuinfo
cat /proc/meminfo

# bootstrap compiler GOROOT
%if !%{golang_bootstrap}
export GOROOT_BOOTSTRAP=/
%else
export GOROOT_BOOTSTRAP=%{goroot_bootstrap_dir}
%endif

# set up final install location
export GOROOT_FINAL=%{goroot}

export GOHOSTOS=linux
export GOHOSTARCH=%{gohostarch}

pushd src
# use our gcc options for this build, but store gcc as default for compiler
export CFLAGS="$RPM_OPT_FLAGS"
export LDFLAGS="$RPM_LD_FLAGS"
export CC="gcc"
export CC_FOR_TARGET="gcc"
export GOOS=linux
export GOARCH=%{gohostarch}
%if !%{external_linker}
export GO_LDFLAGS="-linkmode internal"
%endif
%if !%{cgo_enabled}
export CGO_ENABLED=0
%endif
./make.bash --no-clean
popd

# build shared std lib
%if %{shared}
GOROOT=$(pwd) PATH=$(pwd)/bin:$PATH go install -buildmode=shared std
%endif

%if %{race}
GOROOT=$(pwd) PATH=$(pwd)/bin:$PATH go install -race std
%endif

%install
# create the top level directories
mkdir -p %{buildroot}%{_bindir}
mkdir -p %{buildroot}%{goroot}

# install everything into libdir (until symlink problems are fixed)
# https://code.google.com/p/go/issues/detail?id=5830
cp -apv api bin doc favicon.ico lib pkg robots.txt src misc test VERSION \
   %{buildroot}%{goroot}

# bz1099206
find %{buildroot}%{goroot}/src -exec touch -r %{buildroot}%{goroot}/VERSION "{}" \;
# and level out all the built archives
touch %{buildroot}%{goroot}/pkg
find %{buildroot}%{goroot}/pkg -exec touch -r %{buildroot}%{goroot}/pkg "{}" \;
# generate the spec file ownership of this source tree and packages
cwd=$(pwd)
src_list=$cwd/go-src.list
pkg_list=$cwd/go-pkg.list
shared_list=$cwd/go-shared.list
misc_list=$cwd/go-misc.list
docs_list=$cwd/go-docs.list
tests_list=$cwd/go-tests.list
rm -f $src_list $pkg_list $docs_list $misc_list $tests_list $shared_list
touch $src_list $pkg_list $docs_list $misc_list $tests_list $shared_list
pushd %{buildroot}%{goroot}
    find src/ -type d -a \( ! -name testdata -a ! -ipath '*/testdata/*' \) -printf '%%%dir %{goroot}/%p\n' >> $src_list
    find src/ ! -type d -a \( ! -ipath '*/testdata/*' -a ! -name '*_test.go' \) -printf '%{goroot}/%p\n' >> $src_list

    find bin/ pkg/ -type d -a ! -path '*_dynlink/*' -printf '%%%dir %{goroot}/%p\n' >> $pkg_list
    find bin/ pkg/ ! -type d -a ! -path '*_dynlink/*' -printf '%{goroot}/%p\n' >> $pkg_list

    find doc/ -type d -printf '%%%dir %{goroot}/%p\n' >> $docs_list
    find doc/ ! -type d -printf '%{goroot}/%p\n' >> $docs_list

    find misc/ -type d -printf '%%%dir %{goroot}/%p\n' >> $misc_list
    find misc/ ! -type d -printf '%{goroot}/%p\n' >> $misc_list

%if %{shared}
    find pkg/*_dynlink/ -type d -printf '%%%dir %{goroot}/%p\n' >> $shared_list
    find pkg/*_dynlink/ ! -type d -printf '%{goroot}/%p\n' >> $shared_list
%endif

    find test/ -type d -printf '%%%dir %{goroot}/%p\n' >> $tests_list
    find test/ ! -type d -printf '%{goroot}/%p\n' >> $tests_list
    find src/ -type d -a \( -name testdata -o -ipath '*/testdata/*' \) -printf '%%%dir %{goroot}/%p\n' >> $tests_list
    find src/ ! -type d -a \( -ipath '*/testdata/*' -o -name '*_test.go' \) -printf '%{goroot}/%p\n' >> $tests_list
    # this is only the zoneinfo.zip
    find lib/ -type d -printf '%%%dir %{goroot}/%p\n' >> $tests_list
    find lib/ ! -type d -printf '%{goroot}/%p\n' >> $tests_list
popd

# remove the doc Makefile
rm -rfv %{buildroot}%{goroot}/doc/Makefile

# put binaries to bindir, linked to the arch we're building,
# leave the arch independent pieces in {goroot}
mkdir -p %{buildroot}%{goroot}/bin/linux_%{gohostarch}
ln -sf %{goroot}/bin/go %{buildroot}%{goroot}/bin/linux_%{gohostarch}/go
ln -sf %{goroot}/bin/gofmt %{buildroot}%{goroot}/bin/linux_%{gohostarch}/gofmt

# ensure these exist and are owned
mkdir -p %{buildroot}%{gopath}/src/github.com
mkdir -p %{buildroot}%{gopath}/src/bitbucket.org
mkdir -p %{buildroot}%{gopath}/src/code.google.com/p
mkdir -p %{buildroot}%{gopath}/src/golang.org/x

# make sure these files exist and point to alternatives
rm -f %{buildroot}%{_bindir}/go
ln -sf /etc/alternatives/go %{buildroot}%{_bindir}/go
rm -f %{buildroot}%{_bindir}/gofmt
ln -sf /etc/alternatives/gofmt %{buildroot}%{_bindir}/gofmt

# gdbinit
mkdir -p %{buildroot}%{_sysconfdir}/gdbinit.d
cp -av %{SOURCE100} %{buildroot}%{_sysconfdir}/gdbinit.d/golang.gdb

# prelink blacklist
mkdir -p %{buildroot}%{_sysconfdir}/prelink.conf.d
cp -av %{SOURCE101} %{buildroot}%{_sysconfdir}/prelink.conf.d/golang.conf

# rpm macros
mkdir -p %{buildroot}%{_rpmconfigdir}/macros.d
cp -av %{SOURCE102} %{buildroot}%{_rpmconfigdir}/macros.d/macros.golang


%check
export GOROOT=$(pwd -P)
export PATH="$GOROOT"/bin:"$PATH"
cd src

export CC="gcc"
export CFLAGS="$RPM_OPT_FLAGS"
export LDFLAGS="$RPM_LD_FLAGS"
%if !%{external_linker}
export GO_LDFLAGS="-linkmode internal"
%endif
%if !%{cgo_enabled} || !%{external_linker}
export CGO_ENABLED=0
%endif

# make sure to not timeout
export GO_TEST_TIMEOUT_SCALE=2

# https://github.com/golang/go/issues/18397
%ifarch %arm
export GOTESTONLY='!^(testplugin|go_test:archive/tar|go_test:archive/zip)$'
%else
export GOTESTONLY='!^(go_test:net|go_test:os/exec)$'
%endif

%if %{fail_on_tests}
./run.bash --no-rebuild -v -v -v -k
%else
./run.bash --no-rebuild -v -v -v -k || :
%endif
cd ..


%post bin
%{_sbindir}/update-alternatives --install %{_bindir}/go \
    go %{goroot}/bin/go 90 \
    --slave %{_bindir}/gofmt gofmt %{goroot}/bin/gofmt

%preun bin
if [ $1 = 0 ]; then
    %{_sbindir}/update-alternatives --remove go %{goroot}/bin/go
fi


%files
%doc AUTHORS CONTRIBUTORS LICENSE PATENTS
# VERSION has to be present in the GOROOT, for `go install std` to work
%doc %{goroot}/VERSION
%dir %{goroot}/doc
%doc %{goroot}/doc/*

# go files
%dir %{goroot}
%exclude %{goroot}/bin/
%exclude %{goroot}/pkg/
%exclude %{goroot}/src/
%exclude %{goroot}/doc/
%exclude %{goroot}/misc/
%{goroot}/*

# ensure directory ownership, so they are cleaned up if empty
%dir %{gopath}
%dir %{gopath}/src
%dir %{gopath}/src/github.com/
%dir %{gopath}/src/bitbucket.org/
%dir %{gopath}/src/code.google.com/
%dir %{gopath}/src/code.google.com/p/
%dir %{gopath}/src/golang.org
%dir %{gopath}/src/golang.org/x


# gdbinit (for gdb debugging)
%{_sysconfdir}/gdbinit.d
%{_rpmconfigdir}/macros.d/macros.golang

# prelink blacklist
%{_sysconfdir}/prelink.conf.d

%files -f go-src.list src

%files -f go-docs.list docs

%files -f go-misc.list misc

%files -f go-tests.list tests

%files -f go-pkg.list bin
%{_bindir}/go
%{_bindir}/gofmt

%if %{shared}
%files -f go-shared.list shared
%endif


%changelog
* Thu Sep 05 2019 bcornec <bcornec> 1.11.13-1.mga6
+ Revision: 1437451
- Update to 1.11.13 upstream to fix #25372
- Fix #24014 CVE-2018-1687[3-5] and CVE-2019-6486 with upstream golang 1.11.5
- update to upstream go 1.11.1 for docker 18.06 to compile
- Fix mga#22812 by adding Fedora patch to the CVE
- Fix mga#22646 by upgrading goloang to 1.9.4
- Still fighting with golang tests not passing :-(
- Avoid the go_test:archive/tar also for non-arm systems as it fails
- Put static libc back for golang
- Update golang to 1.8.4 to (mga#21857)
+ kekepower <kekepower>
- This release fixes issues mentioned in mga#21857
- Let's try golang 1.9.1
- Update to version 1.9.2
- Imported spec file and patches from Cauldron
- Cleaned up the spec file a little
- Rebased a patch tmb pointed me to (Patch0)
- Changed $RPM_BUILD_ROOT to %%{buildroot}
- Removed unused patches that were commented out
- Update to version 1.8.5 (mga#21857)

* Thu Jun 22 2017 bcornec <bcornec> 1.8.1-6.mga6
+ Revision: 1108190
- Adds missing patch file for previous security fix
- Call the patch previously added in build !
- Fix #21103 CVE-2017-8932
+ pterjan <pterjan>
- Disable another test on arm
- Try fixing test exclusion
- Disable a test failing on arm

* Sun May 28 2017 akien <akien> 1.8.1-3.mga6
+ Revision: 1105389
- Fix upstream https://github.com/golang/go/issues/19155 (*_test*.go should be kept)

* Sat May 27 2017 neoclust <neoclust> 1.8.1-2.mga6
+ Revision: 1105099
- Rebuilf for ARM

* Wed Apr 19 2017 bcornec <bcornec> 1.8.1-1.mga6
+ Revision: 1096830
- Update to upstream go 1.8.1

* Sun Mar 12 2017 neoclust <neoclust> 1.6.4-2.mga6
+ Revision: 1092153
- Rebuild on armv7

* Mon Jan 02 2017 bcornec <bcornec> 1.6.4-1.mga6
+ Revision: 1079759
- Update to upstream 1.6.4 + patch to pass tests. This version is however not maintained. Will need to work on 1.7.4, but patches need to be re-written

* Tue Sep 13 2016 bcornec <bcornec> 1.6.3-2.mga6
+ Revision: 1052477
- Update to golang upstream 1.6.3 (which also fixes a CVE vulnerability)

* Sat May 07 2016 joequant <joequant> 1.6.2-7.mga6
+ Revision: 1010404
- fix rpm macros

* Sat May 07 2016 joequant <joequant> 1.6.2-6.mga6
+ Revision: 1010377
- update golang specs

* Wed May 04 2016 joequant <joequant> 1.6.2-5.mga6
+ Revision: 1009310
+ rebuild (emptylog)

* Wed May 04 2016 joequant <joequant> 1.6.2-4.mga6
+ Revision: 1009260
- rebuild to bootstrap with new golang

* Wed May 04 2016 joequant <joequant> 1.6.2-3.mga6
+ Revision: 1009252
+ rebuild (emptylog)

* Wed May 04 2016 joequant <joequant> 1.6.2-2.mga6
+ Revision: 1009239
- upgrade to 1.6.2

* Thu Feb 11 2016 umeabot <umeabot> 1.4.3-2.mga6
+ Revision: 954136
- Mageia 6 Mass Rebuild

* Mon Sep 28 2015 joequant <joequant> 1.4.3-1.mga6
+ Revision: 884364
- upgrade to 1.4.3

* Fri Jul 31 2015 joequant <joequant> 1.4.2-2.mga6
+ Revision: 859974
- add rpm macros to golang

* Thu Jul 02 2015 joequant <joequant> 1.4.2-1.mga6
+ Revision: 849505
- upgrade to 1.4.2

* Wed Oct 15 2014 umeabot <umeabot> 1.3.3-2.mga6
+ Revision: 745995
- Second Mageia 5 Mass Rebuild

* Wed Oct 01 2014 pterjan <pterjan> 1.3.3-1.mga5
+ Revision: 733807
- 1.3.3 (minor fixes)

* Sun Sep 28 2014 bcornec <bcornec> 1.3.2-2.mga5
+ Revision: 731375
- Update to upstream 1.3.2 to fix https://bugs.mageia.org/show_bug.cgi?id=14181

* Tue Sep 16 2014 umeabot <umeabot> 1.3.1-2.mga5
+ Revision: 679852
- Mageia 5 Mass Rebuild

* Sat Aug 30 2014 pterjan <pterjan> 1.3.1-1.mga5
+ Revision: 669346
- 1.3.1

* Wed Jul 02 2014 bcornec <bcornec> 1.3-3.mga5
+ Revision: 642323
- Avoid usage of a local script for building with gcc as it's then used when building other components incorerectly. Use build variables instead.

* Mon Jun 30 2014 bcornec <bcornec> 1.3-2.mga5
+ Revision: 641580
- This is now release 2
- Do not run tests for now as they abort on the build system
- Update to upstream 1.3
- Disable 2 tests that are failing in order to build it

* Mon Oct 21 2013 umeabot <umeabot> 1.1.2-3.mga4
+ Revision: 540590
- Mageia 4 Mass Rebuild

* Wed Oct 09 2013 joequant <joequant> 1.1.2-2.mga4
+ Revision: 494028
- fix vim dependency

* Tue Oct 08 2013 joequant <joequant> 1.1.2-1.mga4
+ Revision: 493484
- more spec fixups
- fixup spec
- imported package golang

